/*
 * interface.c for pcihpview - a pci hotplug viewing program
 *
 * Copyright (c) 2001 Greg Kroah-Hartman <greg@kroah.com>
 * Copyright (c) 2001 IBM Corp.
 *
 *	based on code from usbview which is Copyright (c) 1999, 2000 by
 *	Greg Kroah-Hartman, <greg@kroah.com>
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License as
 *	published by the Free Software Foundation; either version 2 of
 *	the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public
 *	License along with this program; if not, write to the Free
 *	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 *	USA.
 *	(See the included file COPYING)
 */


#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "interface.h"
#include "support.h"
#include "slot.h"

#define CHANGE_POWER		0
#define CHANGE_ATTENTION	1
#define ABOUT			42


static GtkWidget *clist1;
static GtkWidget *scrolled_window_main;

static struct slot *selected_slot = NULL;

static void create_popup_menu (GdkEventButton *event, struct slot *slot);


/* gtk callbacks */
static gboolean on_window_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
	gtk_main_quit();
	return FALSE;
}

static gboolean on_clist_click_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data)
{
	if (event->button == 3) {
		create_popup_menu (event, selected_slot);
		return TRUE;
	}
	return FALSE;
}

static gboolean on_clist_select_row_event (GtkWidget *widget, gint row, gint column, GdkEventButton *event, gpointer user_data)
{
	selected_slot = gtk_clist_get_row_data (GTK_CLIST (clist1), row);
	gtk_container_set_focus_child (GTK_CONTAINER (scrolled_window_main), clist1);
	return TRUE;
}

static void on_menu_item_activate (GtkWidget *widget, gpointer user_data)
{
	int retval;

	switch ((int)(user_data)) {
		case CHANGE_POWER:
			retval = slot_change_power (selected_slot);
			if (retval) {
				ShowMessage ("pcihpview", "Unable to change the power setting.", TRUE);
			}
			break;
		case CHANGE_ATTENTION:
			retval = slot_change_attention (selected_slot);
			if (retval) {
				ShowMessage ("pcihpview", "Unable to change the attention setting.", TRUE);
			}
			break;
		default:
			fprintf (stderr, "unknown menu selection\n");
			retval = -1;
			break;
	}
	if (!retval) {
		retval = slot_read_all ();
		gtk_display_slots ();
	}
}

static void on_about_menu_item_refresh (GtkWidget *widget, gpointer user_data)
{
	slot_read_all();
	gtk_display_slots();
}

static void on_about_menu_item_activate (GtkWidget *widget, gpointer user_data)
{
	show_about_dialog();
}

static void create_popup_menu (GdkEventButton *event, struct slot *slot)
{
	GtkWidget *popup;
	GtkWidget *item;

	popup = gtk_menu_new();

#if 0
	item = gtk_tearoff_menu_item_new ();
	gtk_widget_show (item);
	gtk_menu_append (GTK_MENU (popup), item);
#endif

	if (slot != NULL) {
		item = gtk_menu_item_new_with_label (slot->power ? "Power Off" : "Power On");
		gtk_signal_connect (GTK_OBJECT (item), "activate", GTK_SIGNAL_FUNC (on_menu_item_activate), (gpointer)CHANGE_POWER);
		gtk_menu_append (GTK_MENU (popup), item);
		gtk_widget_show (item);

		item = gtk_menu_item_new_with_label (slot->attention ? "Attention Off" : "Attention On");
		gtk_signal_connect (GTK_OBJECT (item), "activate", GTK_SIGNAL_FUNC (on_menu_item_activate), (gpointer)CHANGE_ATTENTION);
		gtk_menu_append (GTK_MENU (popup), item);
		gtk_widget_show (item);

		/* add a separator */
		item = gtk_menu_item_new ();
		gtk_menu_append (GTK_MENU (popup), item);
		gtk_widget_show (item);
	}
	
	item = gtk_menu_item_new_with_label ("Refresh");
	gtk_signal_connect (GTK_OBJECT (item), "activate", GTK_SIGNAL_FUNC (on_about_menu_item_refresh ), NULL);
	gtk_menu_append (GTK_MENU (popup), item);
	gtk_widget_show (item);

	item = gtk_menu_item_new_with_label ("About");
	gtk_signal_connect (GTK_OBJECT (item), "activate", GTK_SIGNAL_FUNC (on_about_menu_item_activate ), NULL);
	gtk_menu_append (GTK_MENU (popup), item);
	gtk_widget_show (item);

	item = gtk_menu_item_new_with_label ("Quit");
	gtk_signal_connect (GTK_OBJECT (item), "activate", GTK_SIGNAL_FUNC (on_window_delete_event), (gpointer)CHANGE_ATTENTION);
	gtk_menu_append (GTK_MENU (popup), item);
	gtk_widget_show (item);

	gtk_menu_popup (GTK_MENU (popup), NULL, NULL, NULL, NULL, event->button, event->time);
}

GtkWidget *create_main_window (void)
{
	GtkWidget *window_main;
	GtkWidget *label_name;
	GtkWidget *label_power;
	GtkWidget *label_adapter;
	GtkWidget *label_latch;
	GtkWidget *label_attention;

	window_main = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_object_set_data (GTK_OBJECT (window_main), "window_main", window_main);
	gtk_window_set_title (GTK_WINDOW (window_main), "pcihpgui");
	gtk_window_set_default_size (GTK_WINDOW (window_main), 500, 300);

	scrolled_window_main = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_ref (scrolled_window_main);
	gtk_object_set_data_full (GTK_OBJECT (window_main), "scrolled_window_main", scrolled_window_main, (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (scrolled_window_main);
	gtk_container_add (GTK_CONTAINER (window_main), scrolled_window_main);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window_main), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	clist1 = gtk_clist_new (5);
	gtk_widget_ref (clist1);
	gtk_object_set_data_full (GTK_OBJECT (window_main), "clist1", clist1, (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (clist1);
	gtk_container_add (GTK_CONTAINER (scrolled_window_main), clist1);
	gtk_clist_set_column_width (GTK_CLIST (clist1), 0, 80);
	gtk_clist_set_column_width (GTK_CLIST (clist1), 1, 80);
	gtk_clist_set_column_width (GTK_CLIST (clist1), 2, 80);
	gtk_clist_set_column_width (GTK_CLIST (clist1), 3, 80);
	gtk_clist_set_column_width (GTK_CLIST (clist1), 4, 80);
	gtk_clist_column_titles_show (GTK_CLIST (clist1));

	label_name = gtk_label_new ("Slot Name");
	gtk_widget_ref (label_name);
	gtk_object_set_data_full (GTK_OBJECT (window_main), "label_name", label_name, (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label_name);
	gtk_clist_set_column_widget (GTK_CLIST (clist1), 0, label_name);

	label_power = gtk_label_new ("Power");
	gtk_widget_ref (label_power);
	gtk_object_set_data_full (GTK_OBJECT (window_main), "label_power", label_power, (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label_power);
	gtk_clist_set_column_widget (GTK_CLIST (clist1), 1, label_power);

	label_adapter = gtk_label_new ("Adapter");
	gtk_widget_ref (label_adapter);
	gtk_object_set_data_full (GTK_OBJECT (window_main), "label_adapter", label_adapter, (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label_adapter);
	gtk_clist_set_column_widget (GTK_CLIST (clist1), 2, label_adapter);

	label_latch = gtk_label_new ("Latch");
	gtk_widget_ref (label_latch);
	gtk_object_set_data_full (GTK_OBJECT (window_main), "label_latch", label_latch, (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label_latch);
	gtk_clist_set_column_widget (GTK_CLIST (clist1), 3, label_latch);

	label_attention = gtk_label_new ("Attention");
	gtk_widget_ref (label_attention);
	gtk_object_set_data_full (GTK_OBJECT (window_main), "label_attention", label_attention, (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label_attention);
	gtk_clist_set_column_widget (GTK_CLIST (clist1), 4, label_attention);

	gtk_signal_connect (GTK_OBJECT (window_main), "delete_event", GTK_SIGNAL_FUNC (on_window_delete_event), NULL);

	gtk_signal_connect (GTK_OBJECT (clist1), "select_row", GTK_SIGNAL_FUNC (on_clist_select_row_event), NULL);
	gtk_signal_connect (GTK_OBJECT (clist1), "button-press-event", GTK_SIGNAL_FUNC (on_clist_click_event), NULL);
	
	return window_main;
}

void gtk_display_slots (void)
{
	struct slot *slot;
	char *data[5];
	int row = 0;

	slot = slot_get_root();
	if (slot == NULL)
		return;

	gtk_clist_clear (GTK_CLIST (clist1));
	selected_slot = NULL;
	
	while (slot) {
		data[0] = slot->name;
		data[1] = (slot->power) ? "on" : "off";
		data[2] = (slot->adapter) ? "present" : "not present";
		data[3] = (slot->latch) ? "open" : "closed";
		data[4] = (slot->attention) ? "on" : "off";

		gtk_clist_insert (GTK_CLIST (clist1), row, data);
		gtk_clist_set_row_data (GTK_CLIST (clist1), row, slot);
		
		slot = slot->next;
		++row;
	}
}

